/* 
Parallel Fractal Demonstration - ParallelFractalEngine.h
Copyright  2000-1  Dean E. Dauger

****************************************************************
License:

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

The GNU General Public License is available by:
Writing to the Free Software Foundation, Inc., 
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
or http://www.fsf.org/copyleft/gpl.html

****************************************************************

See ParallelFractalEngine.c for additional information
*/

#ifdef __MWERKS__
#ifdef powerc
#define myReal			double_t /* Computationally fastest for PowerPC machines */
#else
#define myReal			long double	/* Computationally fastest for 68k machines */
#endif /*powerc*/
#else
#define myReal			double_t
#endif /*__MWERKS__*/
#define myRealMPIDatatype	MPI_DOUBLE

#define Pi		3.1415926535897932384626433832795028
#define invPi	(1.0/Pi)
#define abs(x)			((x<0)?-x:x)

//Utility structures and #define for manipulating Complex numbers in C.  It's usually pretty fast.  
typedef struct Complex	{
	myReal a,b; 
	}	Complex; 

#define CAdd(s,c,d)	s.a=c.a+d.a;s.b=c.b+d.b; 
#define CAccum(s,d)	s.a+=d.a;s.b+=d.b; 
#define CMult(s,c,d)	tr=c.a*d.a-c.b*d.b;s.b=c.b*d.a+d.b*c.a;s.a=tr; //Needs a dummy myReal tr
#define CMultNAdd(s,c,d,f)	tr=f.a+c.a*d.a-c.b*d.b;s.b=f.b+c.b*d.a+d.b*c.a;s.a=tr; 
#define CScalar(s,c,m)	s.a=c.a*m;s.b=c.b*m;  
#define ExpIPi(s,th)	s.a=cos(Pi*th);s.b=sin(Pi*th); 
#define CMagSq(s)	(s.a*s.a+s.b*s.b)
#define CPhase(s)	atan2(s.b,s.a)


#define SendPixelData	1

/* Set default window (and GWorld) size */
/*
#define xSize		256L
#define ySize		256L
/*/
#define xSize		768L
#define ySize		512L
/**/

/* Defines mapping from floating-point number to color */ 
#define ColorMapMultiple		9.125

//Size of one row of 32-bit pixel data in bytes
#ifdef SendPixelData
#define PackedRowBytes		(xSize*sizeof(long))
#endif

/*
//Basic Fractal settings (zoomed out)
#define MaxCount	256L
#define centerA		(-0.55)
#define centerB		(0)
#define radiusA		(1.5)
/*/

// Fractal settings - One of my personal favorites
#define MaxCount	4096L
#define centerA		(-0.7485432130)
#define centerB		(-0.0737514119)
#define radiusA		 (0.0000000500)
/**/

/* Dean's favorite Fractal settings
#define MaxCount	4096L
#define centerA		(-0.7485432130)
#define centerB		(-0.0737514119)
#define radiusA		 (0.0000000500)
*/

/* Nasty Fractal settings - near the limit of double-precision
#define MaxCount	(1L<<18)
#define centerA		(-0.74854321230811)
#define centerB		(-0.07375141141806)
#define radiusA		 (0.0000000000010)
#undef ColorMapMultiple
#define ColorMapMultiple		2.0
*/

/* Pushing numerical envelope Fractal settings
#define MaxCount	(1L<<18)
#define centerA		(-0.74854321230811)
#define centerB		(-0.07375141141806)
#define radiusA		 (0.00000000000001)
#undef ColorMapMultiple
#define ColorMapMultiple		1.0
*/

/*
#define MaxCount	1024L
#define centerA		(-0.7458499)
#define centerB		(-0.1050800)
#define radiusA		 (0.00002)
*/

#define TicksPerCall	30L		/* ComputeTask set point: 30 Ticks = 0.5 seconds */ 

void DoTask(long message); //The only routine Main needs to know about
void AutoLaunchOntoNodes(long numNodes); 
